#!/bin/sh
# r22
case "$-" in *i*) return ;; esac
export LC_ALL=C
#exec 2>&-
APP="Talsyntes"
DIR=~/".talsyntes"
SCRIPT="$DIR/talsyntes.sh"
DATA_DIR="$DIR/data"
VOICES_DIR="$DIR/voices"
SETTINGS_DIR="$DIR/settings"

mkdir -p "$DATA_DIR" "$VOICES_DIR" "$SETTINGS_DIR"

if test "x$SCRIPT" != "x$0" && cp -f "$0" "$SCRIPT" 2>&-
then
   iconv -f utf-8 -t iso8859-1 "$0" >/dev/null 2>&- ||
   iconv -f iso8859-1 -t utf-8 "$0" >"$SCRIPT"
   chmod +x "$SCRIPT" && exec "$SCRIPT" "$@"
fi
get_dialog() {
   which $(cat "$SETTINGS_DIR"/dialog 2>&-) 2>&- ||
   which zenity 2>&- ||
   which Xdialog 2>&- ||
   echo "$0: 'zenity' eller 'Xdialog' saknas" >&2
}
case "$1" in
stop|start|reset|prev|next|restart|loop)
   pgrep -P $(pgrep -u"$(id -u)" -d, "${SCRIPT##*/}") mplayer |
   while read pid
   do kill $(ps -oppid= "$pid") "$pid"
   done
;;
pause)
   find "$DIR" -name mplayer.\*.fifo -type p -exec sh -c 'echo pause >{}' \;
;;
list)
   dialog="$(get_dialog)"
   ps -opid=,ppid=,args= $(pgrep -u"$(id -u)" "${SCRIPT##*/}") |
   grep -vE "(^|[[:blank:]]+)$$([[:blank:]]+|$)" | grep "$*$" |
   while read pid
   do pkill -P "${pid%% *}" "${dialog##*/}"
   done
;;
*)
   ps -opid=,ppid=,args= $(pgrep -u"$(id -u)" "${SCRIPT##*/}") |
   grep -vE "(^|[[:blank:]]+)$$([[:blank:]]+|$)" |
   grep -q "$*$" && exit
;;
esac

create_dialog() {
   dialog="$(get_dialog)"
   case "$1" in
   list)
      type="$2"
      title="$3"
      while read item
      do echo "$item" | grep -q "\($4\)" && default=true || default=false
         case "$dialog" in
         *zenity)
            set "$@" "$default" "${item##*/}"
         ;;
         *Xdialog)
            set "$@" "${item##*/}" "${item##*/}"
            "$default" && set "$@" on || set "$@" off
         ;;
         esac
      done
      shift 4
      case "$dialog" in
      *zenity)
         set -- --width 400 --height 500 --list \
            --"$type"list --text "$title" --column "" --column "" "$@"
      ;;
      *Xdialog)
         set -- --stdout --separator "|" --no-tags \
            --"$type"list "$title" 400x500 0 "$@"
      ;;
      esac
   ;;
   scale)
      case "$dialog" in
      *zenity)
         set -- --scale --text "$2" --print-partial \
            --min-value -100 --max-value 100 --value "$3"
      ;;
      *Xdialog)
         set -- --stdout --rangebox "$2" 10 60 -100 100 "$3"
      ;;
      esac
   ;;
   progress)
      case "$dialog" in
      *zenity)
         set -- --width 300 --progress --auto-close --text "$2"
      ;;
      *Xdialog)
         set -- --stdout --gauge "$2" 5 60
      ;;
      esac
   ;;
   edit)
      case "$dialog" in
      *zenity)
         set --  --text-info --editable
      ;;
      *Xdialog)
         set -- --editbox /dev/null 30 50
      ;;
      esac
   ;;
   question)
      case "$dialog" in
      *zenity)
         set -- --question --text "$2"
      ;;
      *Xdialog)
         set -- --stdout --yesno "$2" 0 0
      ;;
      esac
   ;;
   info|error)
      case "$dialog" in
      *zenity)
         set -- --"$1" --no-wrap --text "$2"
      ;;
      *Xdialog)
         set -- --stdout --fixed-font --msgbox "$2" 0 0
      ;;
      esac
   ;;
   *) return ;;
   esac
   #GTK2_RC_FILES="$DIR"/gtkrc \
   LC_ALL=sv_SE.UTF-8 "${dialog:-:}" --title "$APP" "$@"
}
scale_mapping() {
   awk -vmin="$1" -vdef="$2" -vmax="$3" -vbend="${4:-1}" '
      function f(x)
      {
         factor = (a != b) ? 2*a*b*log(a/b)/(a-b) : (a+b)
         return factor * log(x) * x / (x^2-1) - ((a < b) ? a : b) / bend
      }
      BEGIN {
         a = (max - def)
         b = (def - min)
         if (max < min) { a = -a; b = -b; }
         x0=2; x1=3; y0=f(x0); y1=f(x1); lim=1e-4
         while( x1>1 && ( (x1-x0)>lim || (x0-x1)>lim) ) {
            y1 = f(x1); x2 = x1 - (x1 - x0) * y1 / (y1 - y0)
            x0 = x1; x1 = x2; y0 = y1;
         }
         incl = (x1>1) ? x1 : 1
      }
      {
         expo = $1 / 100
         if (incl != 1) expo = (incl^expo - 1/incl^expo) / (incl - 1/incl)
         expo = (expo + 1) / 2
         if (a != b) expo = ((a / b)^(2 * expo) - 1) / ((a / b)^2 - 1)
         printf("%f\n", min + (max - min) * expo)
      }
   '
   return 0
}

get_amplit() {
   2>&- scale_mapping 0 100 200 <"$SETTINGS_DIR"/amplit || echo 100
}
get_wordgap() {
   2>&- scale_mapping -50 0 50 <"$SETTINGS_DIR"/wordgap || echo 0
}
get_pitch() {
   2>&- scale_mapping 0 50 99 <"$SETTINGS_DIR"/pitch || echo 50
}
get_espeed() {
   2>&- scale_mapping 80 170 370 <"$SETTINGS_DIR"/espeed || echo 170
}
get_speed() {
   2>&- scale_mapping 5 1 0.2 <"$SETTINGS_DIR"/speed || echo 1.0
}
get_freq() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/freq || echo 1.0
}
get_volume() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/volume || echo 1.0
}
get_rate() {
   2>&- scale_mapping 1000 16000 48000 <"$SETTINGS_DIR"/rate || echo 16000
}
get_scale() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/scale || echo 1.0
}
get_stride() {
   2>&- scale_mapping 1 60 400 <"$SETTINGS_DIR"/stride || echo 60
}
get_overlap() {
   2>&- scale_mapping 0 0.2 1 <"$SETTINGS_DIR"/overlap || echo 0.2
}
get_search() {
   2>&- scale_mapping 0 14 200 <"$SETTINGS_DIR"/search || echo 14
}
get_method() {
   2>&- cat "$SETTINGS_DIR"/method || echo tempo
}
get_accel() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/accel || echo 1.0
}
get_voice() {
   2>&- readlink "$SETTINGS_DIR"/voice
}

find -L "$SETTINGS_DIR" -maxdepth 1 -name voice -type l -exec rm {} \;
ls "$VOICES_DIR"/[a-z][a-z][1-9]:* >/dev/null 2>&- &&
test -x "$SCRIPT" &&
test -x "$DIR/mbrola" ||
set install

cd "$DIR"
case "$1" in
stop|pause)
;;
prev)
   line=$(cat "$DIR"/line 2>&-)
   expr "${line:-1}" - 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
next)
   line=$(cat "$DIR"/line 2>&-)
   expr "${line:-1}" + 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
restart)
   echo 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
loop)
   while "$SCRIPT" restart ; do :; done; exit
;;
start)
   CLIP=$(which xclip 2>&- || which xsel 2>&-) &&
   SPEAK=$(which espeak 2>&-) &&
   PLAYER=$(which mplayer 2>&-) ||
   exec "$SCRIPT" install
   test -h "$SETTINGS_DIR/voice" || exec "$SCRIPT" adjust voice
   echo ${SCALETEMPO:-true} >"$SETTINGS_DIR"/scaletempo
   find "$DIR" -name mplayer.\*.fifo -type p -exec rm -f {} \;
   FIFO="$DIR/mplayer.$$.fifo"
   TMP="$(mktemp)"
   trap 'rm -f "$FIFO" "$TMP"' EXIT
   test -p "$FIFO" || mkfifo "$FIFO"
   if voice=$(ls "$VOICES_DIR/$voice":* 2>&-)
   then shift
   else voice="$(get_voice)"
   fi
   voice="${voice##*/}"
   : ${amplit="$(get_amplit)"}
   : ${wordgap="$(get_wordgap)"}
   : ${pitch="$(get_pitch)"}
   : ${espeed="$(get_espeed)"}
   : ${speed="$(get_speed)"}
   : ${freq="$(get_freq)"}
   : ${volume="$(get_volume)"}
   : ${rate="$(get_rate)"}
   : ${accel="$(get_accel)"}

   scaletempo="-speed ${accel:-1}"
   if $SCALETEMPO
   then
      : ${scale="$(get_scale)"}
      : ${stride="$(get_stride)"}
      : ${overlap="$(get_overlap)"}
      : ${search="$(get_search)"}
      : ${method="$(get_method)"}

      scaletempo="$scaletempo -af scaletempo="
      scaletempo="$scaletempo${scaletempo+:}scale=${scale:-1}"
      scaletempo="$scaletempo${scaletempo+:}stride=${stride:-60}"
      scaletempo="$scaletempo${scaletempo+:}overlap=${overlap:-0.20}"
      scaletempo="$scaletempo${scaletempo+:}search=${search:-14}"
      scaletempo="$scaletempo${scaletempo+:}speed=${method:-tempo}"
   fi

#   freq=$(awk '{print(0.953-(exp($1/19.3)-1)/278.66)}' <"$SETTINGS_DIR"/speed)
#   rate=$(awk '{print(17229+(exp($1/19.3)-1)*100)}' <"$SETTINGS_DIR"/speed)

   ${CLIP:-:} -o |
   awk -F'[ \t\n]*[?.,!][ \t\n]' '{
      gsub("\\u[0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f]","")
      gsub("[ \t](and|or)[ \t\n]","\n&")
      for(i=1;i<=NF;i++) {
         printf(" %s ", $i)
         if(i==NF || match($(i+1),"[ \t]")) {
            printf("\n")
         }
      }
   }' >"$TMP"
   line=$(cat "$DIR"/line 2>&-)
   test -s "$TMP" &&
   if cmp -s "$TMP" "$DIR"/text
   then :
   else mv "$TMP" "$DIR"/text && line=1
   fi
   test "${line:-1}" -gt $(wc -l <"$DIR"/text) ||
   test "${line:-1}" -lt 1 && line=1
   sed -n "${line:-1},\$p" "$DIR"/text |
   while read sentence
   do echo "${line:-1}) $sentence"
      echo "$sentence" |
      ${SPEAK:-:} \
         -a "${amplit:-100}" \
         -g "${wordgap:-0}" \
         -p "${pitch:-50}" \
         -s "${espeed:-170}" \
         -v mb-"${voice%:*}" |
      "$DIR/mbrola" -e \
         -t "${speed:-1}" \
         -f "${freq:-1}" \
         -v "${volume:-1}" \
         -l "${rate:-16000}" \
         "$VOICES_DIR/$voice" - -.wav |
      ${PLAYER:-:} \
         -really-quiet -input file="$FIFO" $scaletempo -demuxer audio - ||
      exec env SCALETEMPO=false "$SCRIPT" start
      line=$(expr "${line:-1}" + 1)
      echo "$line" >"$DIR"/line
   done
;;
help)
   help=$(
      sed -n "
         \|^\"$SCRIPT help\"$|{
            n;s/$/:\tLista över tangentkommandon/p}
         \|^\"$SCRIPT pause\"$|{
            n;s/$/:\tPausa uppläsningen/p}
         \|^\"$SCRIPT start\"$|{
            n;s/$/:\tStarta om nuvarande mening/p}
         \|^\"$SCRIPT prev\"$|{
            n;s/$/:\tStarta om föregående mening/p}
         \|^\"$SCRIPT next\"$|{
            n;s/$/:\tHoppa till nästa mening/p}
         \|^\"$SCRIPT voice\"$|{
            n;s/$/:\tVälj röst och språk/p}
         \|^\"$SCRIPT speed\"$|{
            n;s/$/:\tJustera uppläsningsfarten/p}
         \|^\"voice=en1\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening med engelsk röst/p}
         \|^\"voice=sw1\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening med svensk röst/p}
         \|^\"speed=1\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i normal fart/p}
         \|^\"speed=0.75\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i snabb fart/p}
         \|^\"speed=0.5\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i mycket snabb fart/p}
         \|^\"$SCRIPT install\"$|{
            n;s/$/:\tInstallera fler röster/p}
         \|^\"$SCRIPT stop\"$|{
            n;s/$/:\tStoppa uppläsningen/p}
         \|^\"$SCRIPT restart\"$|{
            n;s/$/:\tStarta om hela markeringen/p}
         \|^\"$SCRIPT speed -10\"$|{
            n;s/$/:\tMinska uppläsningsfarten/p}
         \|^\"$SCRIPT speed +10\"$|{
            n;s/$/:\tÖka uppläsningsfarten/p}
         \|^\"$SCRIPT adjust\"$|{
            n;s/$/:\tJustera röstparametrar/p}
         \|^\"$SCRIPT accel\"$|{
            n;s/$/:\tJustera röstparametern 'accel'/p}
         \|^\"$SCRIPT volume\"$|{
            n;s/$/:\tJustera uppläsningsvolymen/p}
         \|^\"$SCRIPT scale\"$|{
            n;s/$/:\tJustera röstparametern 'scale'/p}
         \|^\"$SCRIPT stride\"$|{
            n;s/$/:\tJustera röstparametern 'stride'/p}
         \|^\"$SCRIPT search\"$|{
            n;s/$/:\tJustera röstparametern 'search'/p}
         \|^\"$SCRIPT overlap\"$|{
            n;s/$/:\tJustera röstparametern 'overlap'/p}
         \|^\"$SCRIPT list\"$|{
            n;s/$/:\tLista alla röstparametrar/p}
         \|^\"$SCRIPT reset\"$|{
            n;s/$/:\tNollställ alla röstparametrar/p}
         \|^\"$SCRIPT loop\"$|{
            n;s/$/:\tRepitera nuvarande mening/p}
         \|^\"$SCRIPT dialog\"$|{
            n;s/$/:\tÄndra dialogrutesystem/p}
         \|^\"$SCRIPT method\"$|{
            n;s/$/:\tJustera röstparametern 'method'/p}
         \|^\"$SCRIPT import\"$|{
            n;s/$/:\tImportera en röstparameterlista/p}
         \|^\"$SCRIPT espeed\"$|{
            n;s/$/:\tJustera röstparametern 'espeed'/p}
         \|^\"$SCRIPT amplit\"$|{
            n;s/$/:\tJustera röstparametern 'amplit'/p}
         \|^\"$SCRIPT freq\"$|{
            n;s/$/:\tJustera röstläget/p}
         \|^\"$SCRIPT rate\"$|{
            n;s/$/:\tJustera rösttätheten/p}
         \|^\"$SCRIPT pitch\"$|{
            n;s/$/:\tJustera röstparametern 'pitch'/p}
         \|^\"$SCRIPT wordgap\"$|{
            n;s/$/:\tJustera röstparametern 'wordgap'/p}
         \|^\"$SCRIPT uninstall\"$|{
            n;s/$/:\tAvinstallera $APP/p}
      " <"$DIR"/bindings
   )
   $(cat "$SETTINGS_DIR"/scaletempo 2>&-) ||
   help=$(echo "$help" | grep -vE '(scale|stride|overlap|search|method)')
   help=$(
      echo "Tangentkommandon"
      echo
      echo "$help" | sed -n 's/^F[0-9]\+:/&\t\t/p' | sort -tF -nk2
      echo "$help" | sed -n '/^Shift\s\++\s\+F[0-9]\+:/p' | sort -tF -nk2
      echo "$help" | sed -n 's/^Alt\s\++\s\+F[0-9]:/&\t/p' | sort -tF -nk2
      echo "$help" | sed -n '/^Alt\s\++\s\+F[0-9][0-9]:/p' | sort -tF -nk2
      echo "$help" | grep -vE '^(F[0-9]+:|Shift|Alt)[[:blank:]]' | sort
   )
   create_dialog question "$help"
;;
list)
   cd "$SETTINGS_DIR"
   OPTIONS="amplit wordgap pitch espeed speed freq rate volume accel"
   $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
   OPTIONS="$OPTIONS scale stride overlap search method"
   list=$(
      echo "Nuvarande inställningar"
      echo
      for option in voice $OPTIONS dialog
      do if test -h "$option"
         then
            setting=$(readlink "$option")
            setting="${setting##*/}" && value=""
         else
            setting="$(cat $option 2>&-)"
            value="$(get_$option)"
         fi
         test "$option" = dialog &&
         setting="${value##*/}" && value=""
         test "$option" = method &&
         setting="${setting:-tempo}" && value=""
         printf "%s:\t\t%s\t\t%s\n" \
            "$option" "${setting:-0}" "${value:+(}$value${value:+)}"
      done #| sort
   )
   create_dialog question "$list"
;;
import)
   cd "$SETTINGS_DIR"
   create_dialog edit |
   while read option setting real_value
   do case "${option%:}" in
      voice)
         "$SCRIPT" voice "${setting%:}"
      ;;
      amplit|wordgap|pitch|espeed|speed|freq|rate|volume|accel|dialog)
         echo "$setting" >"${option%:}"
      ;;
      scale|stride|overlap|search|method)
         $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
         echo "$setting" >"${option%:}"
      ;;
      esac
   done
   exec "$SCRIPT" list
;;
reset)
   cd "$SETTINGS_DIR"
   case "$2" in
   '')
      create_dialog question "Återställ standardinställningarna?" || exit
      rm -f amplit wordgap pitch espeed dialog speed freq volume \
         rate scale stride overlap search method accel
   ;;
   amplit|wordgap|pitch|espeed|dialog|speed|freq|volume|rate|accel)
      rm -f "$2"
   ;;
   scale|stride|overlap|search|method)
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) && rm -f "$2"
   ;;
   esac
   pkill -9 "${SCRIPT##*/}"
;;
adjust)
   case "$2" in
   '')
      OPTIONS="amplit wordgap pitch espeed speed freq volume rate accel"
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
      OPTIONS="$OPTIONS scale stride overlap search method"
      set "$1" $(
         printf "%s\n" voice $OPTIONS dialog |
         create_dialog list radio "Välj inställning" '^speed$'
      )
      test $# -gt 1 && exec "$SCRIPT" "$@" || exit
   ;;
   scale|stride|overlap|search|method)
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) || exit
   ;;
   esac
   while :
   do case "$3" in
      voice|dialog|amplit|wordgap|pitch|espeed|speed|freq|volume|rate|accel)
         "$SCRIPT" "$2" & shift
      ;;
      scale|stride|overlap|search|method)
         $(cat "$SETTINGS_DIR"/scaletempo 2>&-) || break
         "$SCRIPT" "$2" & shift
      ;;
      *) break ;;
      esac
   done
   value=$(cat "$SETTINGS_DIR/$2" 2>&-)
   case "$2" in
   voice)
      case "$3" in
      '')
         voice=$(readlink "$SETTINGS_DIR"/voice)
         voice=$(
            ls "$VOICES_DIR"/[a-z][a-z][1-9][:-]* 2>&- |
            create_dialog list radio "Välj röst" "^${voice#../}$"
         ) && test -n "$voice" && voice="$VOICES_DIR/$voice"
      ;;
      *)
         voice=$(ls "$VOICES_DIR/$3":* 2>&-)
      ;;
      esac &&
      ln -sf "$voice" "$SETTINGS_DIR"/voice
      exit
   ;;
   dialog)
      case "$3" in
      '')
         value=$(
            which zenity Xdialog 2>&- |
            create_dialog list radio "Välj dialogsystem" "/${value:-zenity}$"
         ) && echo "$value" >"$SETTINGS_DIR/$2"
      ;;
      zenity|Xdialog)
         echo "$3" >"$SETTINGS_DIR/$2"
      ;;
      esac
      exit
   ;;
   amplit|wordgap|pitch|espeed) title="espeak: $2" ;;
   speed) title="Uppläsningshastighet" ;;
   freq) title="Röstläge" ;;
   volume) title="Uppläsningsvolym" ;;
   rate) title="Rösttäthet" ;;
   accel) title="mplayer: $2" ;;
   scale|stride|overlap|search) title="scaletempo: $2" ;;
   method)
      case "$3" in
      '')
         value=$(
            printf "%s\n" tempo pitch both none |
            create_dialog list radio "scaletempo: $2" "^${value:-tempo}$"
         ) && echo "$value" >"$SETTINGS_DIR/$2"
      ;;
      tempo|pitch|both|none)
         echo "$3" >"$SETTINGS_DIR/$2"
      ;;
      esac
      exit
   ;;
   *) exit
   ;;
   esac
   case "$3" in
   '')
      { create_dialog scale "$title" "${value:-0}" || echo "$value"; } |
      while read value
      do test -n "$value" && echo "$value" >"$SETTINGS_DIR/$2"
      done
   ;;
   *)
      echo "$3" |
      awk -vvalue="$value" '{
         value = ($1) ? value + $1 : 0
         value = (value < 100) ? value : 100
         value = (value > -100) ? value : -100
         print(value)
      }' >"$SETTINGS_DIR/$2"
   ;;
   esac
;;
install)
   #echo 'gtk-font-name = "Sans 20"' >"$DIR"/gtkrc
   dep=$(
      for dep in xbindkeys espeak 'xsel xclip' mplayer
      do for dep in $dep
         do which "$dep" >/dev/null 2>&- && break
         done || echo "$dep"
      done
   )
   if test -n "$dep"
   then
      create_dialog question "Följande saknas:\n\n$dep\n\nInstallera?" &&
      gksudo -- apt-get -q -y install $dep &&
      exec "$SCRIPT" install || exit
   fi
   URL="http://tcts.fpms.ac.be/synthesis/mbrola"
   TMP=$(mktemp)
   trap 'rm -f "$TMP"' EXIT
   wget -qO- "$URL"/mbrcopybin.html |
   sed -n '
      s|.*"\(bin/[^"]*.zip\)"[^>]*>[^<]*LINUX i386.*|\1 (0.3Mb)|p
      s|.*"\(dba/[^/]*/[^"]*\)"[^>]*>\([^:]*:\s*[^<]*\).*|\1\t\2|p
   ' >"$TMP"
   voices=$(
      cd /usr/share/espeak-data/voices/mb &&
      echo mb-[a-z][a-z][1-9] |
      sed 's/mb-//g;s/\(.\)\s\(.\)/\1|\2/g'
   )
   voices=$(
      grep -E "/($voices)/" <"$TMP" | cut -f2 |
      create_dialog list check "Välj språk" '^\(sw\|us\|en\)[[:digit:]]:' |
      sed 's/:[^|]*//g'
   )
   test -n "$voices" || exit
   rm -f "$VOICES_DIR"/[a-z][a-z][1-9][:-]* 2>&-
   cd "$DATA_DIR"
   grep -E "(^bin|/$voices/)" <"$TMP" |
   awk '{
      size[NR]=size[NR-1]+substr($NF,2,length($NF)-4)
      $NF=""
      voice[NR]=$0
   } END {
      for(i=1; i<=NR; i++) {
         print(100*size[i]/size[NR],voice[i])
      }
   }' |
   while read progress file voice
   do test -f "${file##*/}" ||
      wget -q "$URL/$file" 2>&1
      if test -z "$voice"
      then
         unzip -p "${file##*/}" mbrola-linux-i386 >"$DIR"/mbrola
         chmod +x "$DIR"/mbrola
      else
         unzip -p "${file##*/}" "${voice%%:*}" >"$VOICES_DIR/$voice" 2>&- ||
         unzip -p "${file##*/}" "${voice%%:*}/${voice%%:*}" \
            >"$VOICES_DIR/$voice"
         test -e /usr/share/espeak-data/voices/mb/mb-"${voice%%:*}"-en &&
         ln -s "$voice" "$VOICES_DIR/${voice%:*}-en:${voice#*:} (English)"
      fi
      echo $progress
   done |
   create_dialog progress "Hämtar Röster..."
   test -e "$DIR"/bindings |
   printf '"%s"\n%s\n\n' \
                   "$SCRIPT help"       "F1" \
                   "$SCRIPT pause"      "F2" \
                   "$SCRIPT start"      "F3" \
                   "$SCRIPT prev"       "F4" \
                   "$SCRIPT next"       "F5" \
                   "$SCRIPT voice"      "F6" \
                   "$SCRIPT speed"      "F7" \
      "voice=en1    $SCRIPT start"      "F8" \
      "voice=sw1    $SCRIPT start"      "F9" \
      "speed=1      $SCRIPT start"      "F10" \
      "speed=0.75   $SCRIPT start"      "F11" \
      "speed=0.5    $SCRIPT start"      "F12" \
                   "$SCRIPT install"    "Shift + F1" \
                   "$SCRIPT stop"       "Shift + F2" \
                   "$SCRIPT restart"    "Shift + F3" \
                   "$SCRIPT speed -10"  "Shift + F4" \
                   "$SCRIPT speed +10"  "Shift + F5" \
                   "$SCRIPT adjust"     "Shift + F6" \
                   "$SCRIPT accel"      "Shift + F7" \
                   "$SCRIPT volume"     "Shift + F8" \
                   "$SCRIPT scale"      "Shift + F9" \
                   "$SCRIPT stride"     "Shift + F10" \
                   "$SCRIPT search"     "Shift + F11" \
                   "$SCRIPT overlap"    "Shift + F12" \
                   "$SCRIPT list"       "Alt + F1" \
                   "$SCRIPT reset"      "Alt + F2" \
                   "$SCRIPT loop"       "Alt + F3" \
                   "$SCRIPT dialog"     "Alt + F4" \
                   "$SCRIPT method"     "Alt + F5" \
                   "$SCRIPT import"     "Alt + F6" \
                   "$SCRIPT espeed"     "Alt + F7" \
                   "$SCRIPT amplit"     "Alt + F8" \
                   "$SCRIPT freq"       "Alt + F9" \
                   "$SCRIPT rate"       "Alt + F10" \
                   "$SCRIPT pitch"      "Alt + F11" \
                   "$SCRIPT wordgap"    "Alt + F12" \
   >"$DIR"/bindings
   test -s ~/.xsession || echo >>~/.xsession
   sed -i "1ixbindkeys -f \"$DIR/bindings\"" ~/.xsession
   ps -opid=,args= $(pgrep -d, xbindkeys) |
   sed -n "s|\s\+xbindkeys -f $DIR.*||p" |
   while read pid
   do kill -9 "$pid"
   done
   setsid xbindkeys -f "$DIR/bindings" &&
   create_dialog info "$APP installerad"
   exec "$SCRIPT" voice
;;
uninstall)
   create_dialog question "Avinstallera $APP?" || exit
   sed -i "\|^xbindkeys -f \"$DIR/bindings\"$|d" ~/.xsession
   rm -rf "$DIR"
   create_dialog info "$APP avinstallerad"
;;
pack)
   REVISION=$(sed -n '2s/^\s*#\s*/-/p' "$0")
   tar cjf "${SCRIPT%.sh}$REVISION.tar.bz2" "${SCRIPT##*/}"
;;
amplit|wordgap|pitch|espeed)
   exec "$SCRIPT" adjust "$@"
;;
voice|dialog|speed|freq|volume|rate|accel)
   exec "$SCRIPT" adjust "$@"
;;
scale|stride|overlap|search|method)
   $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
   exec "$SCRIPT" adjust "$@"
;;
*)
   exec "$SCRIPT" start "$@"
;;
esac
